@@ -6,7 +6,7 @@ from __future__ import annotations
 import os
 import shutil
 from copy import deepcopy
-from typing import Any
+from typing import Any, Literal
 
 import cv2
 import numpy as np
@@ -19,6 +19,7 @@ from maps4fs.generator.component.base.component_image import ImageComponent
 from maps4fs.generator.component.base.component_mesh import MeshComponent
 from maps4fs.generator.component.dem import DEM
 from maps4fs.generator.component.texture import Texture
+from maps4fs.generator.monitor import monitor_performance
 from maps4fs.generator.settings import Parameters
 
 SEGMENT_LENGTH = 2
@@ -38,6 +39,7 @@ class Background(MeshComponent, ImageComponent):
             info, warning. If not provided, default logging will be used.
     """
 
+    @monitor_performance
     def preprocess(self) -> None:
         """Registers the DEMs for the background terrain."""
         self.stl_preview_path = os.path.join(self.previews_directory, "background_dem.stl")
@@ -57,6 +59,14 @@ class Background(MeshComponent, ImageComponent):
         os.makedirs(self.background_directory, exist_ok=True)
         os.makedirs(self.water_directory, exist_ok=True)
 
+        self.textured_mesh_directory = os.path.join(self.background_directory, "textured_mesh")
+        os.makedirs(self.textured_mesh_directory, exist_ok=True)
+
+        self.assets_background_directory = os.path.join(self.map.assets_directory, "background")
+        self.assets_water_directory = os.path.join(self.map.assets_directory, "water")
+        os.makedirs(self.assets_background_directory, exist_ok=True)
+        os.makedirs(self.assets_water_directory, exist_ok=True)
+
         self.water_resources_path = os.path.join(self.water_directory, "water_resources.png")
 
         self.output_path = os.path.join(self.background_directory, f"{Parameters.FULL}.png")
@@ -68,6 +78,9 @@ class Background(MeshComponent, ImageComponent):
             self.background_directory, "not_substracted.png"
         )
         self.not_resized_path: str = os.path.join(self.background_directory, "not_resized.png")
+        self.not_resized_with_foundations_path: str = os.path.join(
+            self.background_directory, "not_resized_with_foundations.png"
+        )
 
         self.flatten_water_to: int | None = None
 
@@ -110,9 +123,30 @@ class Background(MeshComponent, ImageComponent):
 
         if self.map.background_settings.generate_background:
             self.generate_obj_files()
+            if self.game.mesh_processing:
+                self.logger.debug("Mesh processing is enabled, will decimate, texture and convert.")
+                self.decimate_background_mesh()
+                self.texture_background_mesh()
+                background_conversion_result = self.convert_background_mesh_to_i3d()
+                if background_conversion_result:
+                    self.add_note_file(asset="background")
+            else:
+                self.logger.warning(
+                    "Mesh processing is disabled for the game, skipping background mesh processing."
+                )
         if self.map.background_settings.generate_water:
             self.generate_water_resources_obj()
+            if self.game.mesh_processing:
+                self.logger.debug("Mesh processing is enabled, will convert water mesh to i3d.")
+                water_conversion_result = self.convert_water_mesh_to_i3d()
+                if water_conversion_result:
+                    self.add_note_file(asset="water")
+            else:
+                self.logger.warning(
+                    "Mesh processing is disabled for the game, skipping water mesh processing."
+                )
 
+    @monitor_performance
     def create_foundations(self, dem_image: np.ndarray) -> np.ndarray:
         """Creates foundations for buildings based on the DEM data.
 
@@ -218,6 +252,7 @@ class Background(MeshComponent, ImageComponent):
         )
         self.create_qgis_scripts([qgis_layer, qgis_layer_with_margin])
 
+    @monitor_performance
     def generate_obj_files(self) -> None:
         """Iterates over all dems and generates 3D obj files based on DEM data.
         If at least one DEM file is missing, the generation will be stopped at all.
@@ -253,6 +288,261 @@ class Background(MeshComponent, ImageComponent):
             remove_center=self.map.background_settings.remove_center,
         )
 
+    @staticmethod
+    def get_decimate_factor(map_size: int) -> float:
+        """Returns the decimation factor based on the map size.
+
+        Arguments:
+            map_size (int): The size of the map in pixels.
+
+        Raises:
+            ValueError: If the map size is too large for decimation.
+
+        Returns:
+            float -- The decimation factor.
+        """
+        thresholds = {
+            2048: 0.1,
+            4096: 0.05,
+            8192: 0.025,
+            16384: 0.0125,
+        }
+        for threshold, factor in thresholds.items():
+            if map_size <= threshold:
+                return factor
+        raise ValueError(
+            "Map size is too large for decimation, perform manual decimation in Blender."
+        )
+
+    @staticmethod
+    def get_background_texture_resolution(map_size: int) -> int:
+        """Returns the background texture resolution based on the map size.
+
+        Arguments:
+            map_size (int): The size of the map in pixels.
+
+        Returns:
+            int -- The background texture resolution.
+        """
+        resolutions = {
+            2048: 2048,
+            4096: Parameters.MAXIMUM_BACKGROUND_TEXTURE_SIZE,
+            8192: Parameters.MAXIMUM_BACKGROUND_TEXTURE_SIZE,
+            16384: Parameters.MAXIMUM_BACKGROUND_TEXTURE_SIZE,
+        }
+        for threshold, resolution in resolutions.items():
+            if map_size <= threshold:
+                return resolution
+        return Parameters.MAXIMUM_BACKGROUND_TEXTURE_SIZE
+
+    @monitor_performance
+    def decimate_background_mesh(self) -> None:
+        """Decimates the background mesh based on the map size."""
+        if not self.assets.background_mesh or not os.path.isfile(self.assets.background_mesh):
+            self.logger.warning("Background mesh not found, cannot generate i3d background.")
+            return
+
+        try:
+            mesh = trimesh.load_mesh(self.assets.background_mesh, force="mesh")
+        except Exception as e:
+            self.logger.error("Could not load background mesh: %s", e)
+            return
+
+        try:
+            decimate_factor = self.get_decimate_factor(self.map_size)
+        except ValueError as e:
+            self.logger.error("Could not determine decimation factor: %s", e)
+            return
+
+        decimated_save_path = os.path.join(
+            self.background_directory, f"{Parameters.DECIMATED_BACKGROUND}.obj"
+        )
+        try:
+            self.logger.debug("Decimating background mesh with factor %s.", decimate_factor)
+            decimated_mesh = self.decimate_mesh(mesh, decimate_factor)
+            self.logger.debug("Decimation completed.")
+        except Exception as e:
+            self.logger.error("Could not decimate background mesh: %s", e)
+            return
+
+        decimated_mesh.export(decimated_save_path)
+        self.logger.debug("Decimated background mesh saved: %s", decimated_save_path)
+
+        self.assets.decimated_background_mesh = decimated_save_path
+
+    @monitor_performance
+    def texture_background_mesh(self) -> None:
+        """Textures the background mesh using satellite imagery."""
+        satellite_component = self.map.get_satellite_component()
+        if not satellite_component:
+            self.logger.warning("Satellite component not found, cannot texture background mesh.")
+            return
+
+        background_texture_path = satellite_component.assets.background
+
+        if not background_texture_path or not os.path.isfile(background_texture_path):
+            self.logger.warning("Background texture not found, cannot texture background mesh.")
+            return
+
+        decimated_background_mesh_path = self.assets.decimated_background_mesh
+        if not decimated_background_mesh_path or not os.path.isfile(decimated_background_mesh_path):
+            self.logger.warning(
+                "Decimated background mesh not found, cannot texture background mesh."
+            )
+            return
+
+        background_texture_resolution = self.get_background_texture_resolution(self.map_size)
+        non_resized_texture_image = cv2.imread(background_texture_path, cv2.IMREAD_UNCHANGED)
+
+        if non_resized_texture_image is None:
+            self.logger.error(
+                "Failed to read background texture image: %s", background_texture_path
+            )
+            return
+
+        resized_texture_image = cv2.resize(
+            non_resized_texture_image,  # type: ignore
+            (background_texture_resolution, background_texture_resolution),
+            interpolation=cv2.INTER_AREA,
+        )
+
+        resized_texture_save_path = os.path.join(
+            self.textured_mesh_directory,
+            "background_texture.jpg",
+        )
+
+        cv2.imwrite(resized_texture_save_path, resized_texture_image)
+        self.logger.debug("Resized background texture saved: %s", resized_texture_save_path)
+
+        decimated_mesh = trimesh.load_mesh(decimated_background_mesh_path, force="mesh")
+
+        if decimated_mesh is None:
+            self.logger.error("Failed to load decimated mesh after all retry attempts")
+            return
+
+        try:
+            obj_save_path, mtl_save_path = self.texture_mesh(
+                decimated_mesh,
+                resized_texture_save_path,
+                output_directory=self.textured_mesh_directory,
+                output_name="background_textured_mesh",
+            )
+
+            self.assets.textured_background_mesh = obj_save_path
+            self.assets.textured_background_mtl = mtl_save_path
+            self.assets.resized_background_texture = resized_texture_save_path
+            self.logger.debug("Textured background mesh saved: %s", obj_save_path)
+        except Exception as e:
+            self.logger.error("Could not texture background mesh: %s", e)
+            return
+
+    @monitor_performance
+    def convert_background_mesh_to_i3d(self) -> bool:
+        """Converts the textured background mesh to i3d format.
+
+        Returns:
+            bool -- True if the conversion was successful, False otherwise.
+        """
+        if not self.assets.textured_background_mesh or not os.path.isfile(
+            self.assets.textured_background_mesh
+        ):
+            self.logger.warning("Textured background mesh not found, cannot convert to i3d.")
+            return False
+
+        if not self.assets.resized_background_texture or not os.path.isfile(
+            self.assets.resized_background_texture
+        ):
+            self.logger.warning("Resized background texture not found, cannot convert to i3d.")
+            return False
+
+        try:
+            mesh = trimesh.load_mesh(self.assets.textured_background_mesh, force="mesh")
+        except Exception as e:
+            self.logger.error("Could not load textured background mesh: %s", e)
+            return False
+
+        try:
+            i3d_background_terrain = self.mesh_to_i3d(
+                mesh,
+                output_dir=self.assets_background_directory,
+                name=Parameters.BACKGROUND_TERRAIN,
+                texture_path=self.assets.resized_background_texture,
+                water_mesh=False,
+            )
+            self.logger.debug(
+                "Background mesh converted to i3d successfully: %s", i3d_background_terrain
+            )
+            self.assets.background_terrain_i3d = i3d_background_terrain
+            return True
+        except Exception as e:
+            self.logger.error("Could not convert background mesh to i3d: %s", e)
+            return False
+
+    def add_note_file(self, asset: Literal["background", "water"]) -> None:
+        """Adds a note file to the background or water directory.
+
+        Arguments:
+            asset (Literal["background", "water"]): The asset type to add the note file to.
+        """
+        filename = "DO_NOT_USE_THESE_FILES.txt"
+        note_template = (
+            "Please find the ready-to-use {asset} i3d files in the {asset_directory} directory."
+        )
+        directory = {
+            "background": self.background_directory,
+            "water": self.water_directory,
+        }
+
+        content = (
+            "The files in this directory can be used to create the mesh files manually in Blender. "
+            "However, it's recommended to use the ready-to-use i3d files located in the assets "
+            "directory. There you'll find the i3d files, that can be imported directly into the "
+            "Giants Editor without any additional processing."
+        )
+        note = note_template.format(
+            asset=asset,
+            asset_directory=f"assets/{asset}",
+        )
+
+        file_path = os.path.join(directory[asset], filename)
+        with open(file_path, "w", encoding="utf-8") as f:
+            f.write(content + "\n\n" + note)
+
+    @monitor_performance
+    def convert_water_mesh_to_i3d(self) -> bool:
+        """Converts the line-based water mesh to i3d format.
+
+        Returns:
+            bool -- True if the conversion was successful, False otherwise.
+        """
+        if not self.assets.line_based_water_mesh or not os.path.isfile(
+            self.assets.line_based_water_mesh
+        ):
+            self.logger.warning("Line-based water mesh not found, cannot convert to i3d.")
+            return False
+
+        try:
+            mesh = trimesh.load_mesh(self.assets.line_based_water_mesh, force="mesh")
+        except Exception as e:
+            self.logger.error("Could not load line-based water mesh: %s", e)
+            return False
+
+        try:
+            i3d_water_resources = self.mesh_to_i3d(
+                mesh,
+                output_dir=self.assets_water_directory,
+                name=Parameters.WATER_RESOURCES,
+                water_mesh=True,
+            )
+            self.logger.debug(
+                "Water resources mesh converted to i3d successfully: %s", i3d_water_resources
+            )
+            self.assets.water_resources_i3d = i3d_water_resources
+            return True
+        except Exception as e:
+            self.logger.error("Could not convert water mesh to i3d: %s", e)
+            return False
+
     def save_map_dem(self, dem_path: str, save_path: str | None = None) -> str:
         """Cuts out the center of the DEM (the actual map) and saves it as a separate file.
 
@@ -274,6 +564,10 @@ class Background(MeshComponent, ImageComponent):
 
         if self.map.dem_settings.add_foundations:
             dem_data = self.create_foundations(dem_data)
+            cv2.imwrite(self.not_resized_with_foundations_path, dem_data)
+            self.logger.debug(
+                "Not resized DEM with foundations saved: %s", self.not_resized_with_foundations_path
+            )
 
         output_size = self.scaled_size + 1
 
@@ -451,6 +745,7 @@ class Background(MeshComponent, ImageComponent):
         cv2.imwrite(colored_dem_path, dem_data_colored)
         return colored_dem_path
 
+    @monitor_performance
     def create_background_textures(self) -> None:
         """Creates background textures for the map."""
         layers_schema = self.map.texture_schema
@@ -503,6 +798,7 @@ class Background(MeshComponent, ImageComponent):
 
         cv2.imwrite(self.water_resources_path, background_image)
 
+    @monitor_performance
     def subtraction(self) -> None:
         """Subtracts the water depth from the DEM data where the water resources are located."""
         if not self.water_resources_path:
@@ -586,7 +882,7 @@ class Background(MeshComponent, ImageComponent):
                 continue
 
             # Make Polygon a little bit bigger to hide under the terrain when creating water planes.
-            polygon = polygon.buffer(Parameters.WATER_ADD_WIDTH, resolution=4)
+            polygon = polygon.buffer(Parameters.WATER_ADD_WIDTH, quad_segs=4)
 
             polygons.append(polygon)
 
@@ -597,6 +893,7 @@ class Background(MeshComponent, ImageComponent):
                     polygon_points=polygon.exterior.coords,
                     angle=self.rotation,
                     canvas_size=self.background_size,
+                    xshift=-Parameters.BACKGROUND_DISTANCE,
                 )
                 fitted_polygon = shapely.Polygon(fitted_polygon_points)
                 fitted_polygons.append(fitted_polygon)
@@ -622,6 +919,8 @@ class Background(MeshComponent, ImageComponent):
         mesh.export(line_based_save_path)
         self.logger.debug("Line-based water mesh saved to %s", line_based_save_path)
 
+        self.assets.line_based_water_mesh = line_based_save_path
+
     def mesh_from_3d_polygons(
         self, polygons: list[shapely.Polygon], single_z_value: int | None = None
     ) -> Trimesh | None:
@@ -685,6 +984,7 @@ class Background(MeshComponent, ImageComponent):
         mesh = trimesh.Trimesh(vertices=vertices, faces=faces, process=False)
         return mesh
 
+    @monitor_performance
     def generate_water_resources_obj(self) -> None:
         """Generates 3D obj files based on water resources data."""
         self.logger.debug("Starting water resources generation...")
@@ -751,13 +1051,24 @@ class Background(MeshComponent, ImageComponent):
 
         self.plane_from_np(elevated_water, elevated_save_path, include_zeros=False)
 
+    @monitor_performance
     def flatten_roads(self) -> None:
         """Flattens the roads in the DEM data by averaging the height values along the road polylines."""
-        if not self.not_resized_path or not os.path.isfile(self.not_resized_path):
+        supported_files = [self.not_resized_with_foundations_path, self.not_resized_path]
+
+        base_image_path = None
+        for supported_file in supported_files:
+            if not supported_file or not os.path.isfile(supported_file):
+                continue
+
+            base_image_path = supported_file
+            break
+
+        if not base_image_path:
             self.logger.warning("No DEM data found for flattening roads.")
             return
 
-        dem_image = cv2.imread(self.not_resized_path, cv2.IMREAD_UNCHANGED)
+        dem_image = cv2.imread(base_image_path, cv2.IMREAD_UNCHANGED)
         if dem_image is None:
             self.logger.warning("Failed to read DEM data.")
             return
@@ -794,41 +1105,80 @@ class Background(MeshComponent, ImageComponent):
             total_length = polyline.length
             self.logger.debug("Total length of the road polyline: %s", total_length)
 
-            current_distance = 0
-            segments: list[shapely.LineString] = []
-            while current_distance < total_length:
-                start_point = polyline.interpolate(current_distance)
-                end_distance = min(current_distance + SEGMENT_LENGTH, total_length)
-                end_point = polyline.interpolate(end_distance)
+            # Step 1: Create complete road mask once
+            road_mask = np.zeros(dem_image.shape, dtype=np.uint8)
+            # OpenCV thickness is total width, not radius like Shapely buffer
+            # So we need width * 4 to match the old buffer(width * 2) behavior
+            line_thickness = int(width * 4)
 
-                # Create a small segment LineString
-                segment = shapely.LineString([start_point, end_point])
-                segments.append(segment)
-                current_distance += SEGMENT_LENGTH
+            # Get densely sampled points for smooth road
+            dense_sample_distance = min(SEGMENT_LENGTH, total_length / 100)  # At least 100 samples
+            num_dense_points = max(100, int(total_length / dense_sample_distance))
+            dense_distances = np.linspace(0, total_length, num_dense_points)
+            dense_points = [polyline.interpolate(d) for d in dense_distances]
+            dense_coords = np.array([(int(p.x), int(p.y)) for p in dense_points], dtype=np.int32)
 
-            self.logger.debug("Number of segments created: %s", len(segments))
+            # Draw entire road at once
+            if len(dense_coords) > 1:
+                cv2.polylines(road_mask, [dense_coords], False, 255, thickness=line_thickness)
 
-            road_polygons: list[shapely.Polygon] = []
-            for segment in segments:
-                polygon = segment.buffer(
-                    width * 2, resolution=4, cap_style="flat", join_style="mitre"
-                )
-                road_polygons.append(polygon)
+            # Step 2: Get all road pixels that need processing
+            road_pixels = np.where(road_mask == 255)
+            if len(road_pixels[0]) == 0:
+                continue
 
-            for polygon in road_polygons:
-                polygon_points = polygon.exterior.coords
-                road_np = self.polygon_points_to_np(polygon_points)
-                mask = np.zeros(dem_image.shape, dtype=np.uint8)
+            road_y, road_x = road_pixels
+            self.logger.debug("Processing %s road pixels", len(road_y))
 
-                try:
-                    cv2.fillPoly(mask, [road_np], 255)  # type: ignore
-                except Exception as e:
-                    self.logger.debug("Could not create mask for road with error: %s", e)
-                    continue
+            # Step 3: Efficient distance-based smooth gradation
+            # Use much larger segments (10-20x SEGMENT_LENGTH) but interpolate between them
+            large_segment_length = SEGMENT_LENGTH * 15  # 30 units instead of 2
+            num_large_segments = max(1, int(np.ceil(total_length / large_segment_length)))
+            large_distances = np.linspace(0, total_length, num_large_segments + 1)
 
-                mean_value = cv2.mean(dem_image, mask=mask)[0]  # type: ignore
-                dem_image[mask == 255] = mean_value
-                full_mask[mask == 255] = 255
+            # Calculate elevation values at large segment points
+            segment_elevations = []
+            for dist in large_distances:
+                sample_point = polyline.interpolate(dist)
+                sample_x, sample_y = int(sample_point.x), int(sample_point.y)
+
+                # Sample elevation in small area around this point
+                sample_radius = max(5, line_thickness // 4)
+                y_min = max(0, sample_y - sample_radius)
+                y_max = min(dem_image.shape[0], sample_y + sample_radius)
+                x_min = max(0, sample_x - sample_radius)
+                x_max = min(dem_image.shape[1], sample_x + sample_radius)
+
+                if y_max > y_min and x_max > x_min:
+                    sample_elevation = np.mean(dem_image[y_min:y_max, x_min:x_max])  # type: ignore
+                else:
+                    sample_elevation = (
+                        dem_image[sample_y, sample_x]  # type: ignore
+                        if 0 <= sample_y < dem_image.shape[0] and 0 <= sample_x < dem_image.shape[1]
+                        else 0
+                    )
+
+                segment_elevations.append(sample_elevation)
+
+            # Step 4: Interpolate elevations smoothly along entire road
+            road_distances_from_start = []
+            for i in range(len(road_y)):  # pylint: disable=consider-using-enumerate
+                px, py = road_x[i], road_y[i]
+                # Find closest point on polyline (approximation)
+                closest_point = polyline.interpolate(polyline.project(shapely.Point(px, py)))
+                distance_along_road = polyline.project(closest_point)
+                road_distances_from_start.append(distance_along_road)
+
+            road_distances_from_start = np.array(road_distances_from_start)  # type: ignore
+
+            # Interpolate elevation values based on distance along road
+            interpolated_elevations = np.interp(
+                road_distances_from_start, large_distances, segment_elevations
+            )
+
+            # Step 5: Apply interpolated elevations to road pixels
+            dem_image[road_y, road_x] = interpolated_elevations
+            full_mask[road_y, road_x] = 255
 
         main_dem_path = self.game.dem_file_path(self.map_directory)
         dem_image = self.blur_by_mask(dem_image, full_mask, blur_radius=5)
