@@ -10,6 +10,7 @@ from collections import defaultdict
 from typing import Any, Callable, Generator, Optional
 
 import cv2
+import geopandas as gpd
 import numpy as np
 import osmnx as ox
 import pandas as pd
@@ -20,6 +21,7 @@ from tqdm import tqdm
 
 from maps4fs.generator.component.base.component_image import ImageComponent
 from maps4fs.generator.component.layer import Layer
+from maps4fs.generator.monitor import monitor_performance
 from maps4fs.generator.settings import Parameters
 
 
@@ -172,6 +174,7 @@ class Texture(ImageComponent):
         self._read_parameters()
         self.draw()
         self.rotate_textures()
+        self.merge_into()
 
         if not self.kwargs.get("skip_scaling", False):
             self.scale_textures()
@@ -184,6 +187,7 @@ class Texture(ImageComponent):
         for layer in self.layers:
             self.assets[layer.name] = layer.path(self._weights_dir)
 
+    @monitor_performance
     def add_borders(self) -> None:
         """Iterates over all the layers and picks the one which have the border propety defined.
         Borders are distance from the edge of the map on each side (top, right, bottom, left).
@@ -256,6 +260,48 @@ class Texture(ImageComponent):
                     "Procedural file %s copied from %s.", procedural_save_path, texture_paths[0]
                 )
 
+    def get_layer_by_name(self, layer_name: str) -> Layer | None:
+        """Returns the layer with the given name.
+
+        Arguments:
+            layer_name: The name of the layer to retrieve.
+
+        Returns:
+            The layer with the given name, or None if not found.
+        """
+        for layer in self.layers:
+            if layer.name == layer_name:
+                return layer
+        return None
+
+    @monitor_performance
+    def merge_into(self) -> None:
+        """Merges the content of layers into their target layers."""
+        for layer in self.layers:
+            if layer.merge_into:
+                target_layer = self.get_layer_by_name(layer.merge_into)
+                if target_layer:
+                    target_layer_image = cv2.imread(
+                        target_layer.path(self._weights_dir), cv2.IMREAD_UNCHANGED
+                    )
+                    layer_image = cv2.imread(layer.path(self._weights_dir), cv2.IMREAD_UNCHANGED)
+                    if target_layer_image is not None and layer_image is not None:
+                        if target_layer_image.shape != layer_image.shape:
+                            self.logger.warning(
+                                "Layer %s and target layer %s have different shapes, skipping merge.",
+                                layer.name,
+                                target_layer.name,
+                            )
+                            continue
+                        target_layer_image = cv2.add(target_layer_image, layer_image)
+                        cv2.imwrite(target_layer.path(self._weights_dir), target_layer_image)
+                    self.logger.debug("Merged layer %s into %s.", layer.name, target_layer.name)
+
+                    # Clear the content of the layer which have merge_into property.
+                    cv2.imwrite(layer.path(self._weights_dir), np.zeros_like(layer_image))
+                    self.logger.debug("Cleared layer %s.", layer.name)
+
+    @monitor_performance
     def rotate_textures(self) -> None:
         """Rotates textures of the layers which have tags."""
         if self.rotation:
@@ -278,6 +324,7 @@ class Texture(ImageComponent):
                         "Skipping rotation of layer %s because it has no tags.", layer.name
                     )
 
+    @monitor_performance
     def scale_textures(self) -> None:
         """Resizes all the textures to the map output size."""
         if not self.map.output_size:
@@ -322,6 +369,7 @@ class Texture(ImageComponent):
         ]
         return {attr: getattr(self, attr, None) for attr in useful_attributes}
 
+    @monitor_performance
     def _prepare_weights(self):
         self.logger.debug("Starting preparing weights from %s layers.", len(self.layers))
 
@@ -386,6 +434,7 @@ class Texture(ImageComponent):
             ),
         )
 
+    @monitor_performance
     def draw(self) -> None:
         """Iterates over layers and fills them with polygons from OSM data."""
         layers = self.layers_by_priority()
@@ -504,6 +553,7 @@ class Texture(ImageComponent):
                 }
                 info_layer_data[f"{layer.info_layer}_polylines"].append(linestring_entry)  # type: ignore
 
+    @monitor_performance
     def dissolve(self) -> None:
         """Dissolves textures of the layers with tags into sublayers for them to look more
         natural in the game.
@@ -512,46 +562,52 @@ class Texture(ImageComponent):
         files of the corresponding layer and saves the changes to the files.
         """
         for layer in tqdm(self.layers, desc="Dissolving textures", unit="layer"):
-            if not layer.tags:
-                self.logger.debug("Layer %s has no tags, there's nothing to dissolve.", layer.name)
-                continue
-            layer_path = layer.path(self._weights_dir)
-            layer_paths = layer.paths(self._weights_dir)
+            self.dissolve_layer(layer)
 
-            if len(layer_paths) < 2:
-                self.logger.debug("Layer %s has only one texture, skipping.", layer.name)
-                continue
+    def dissolve_layer(self, layer: Layer) -> None:
+        """Dissolves texture of the layer into sublayers."""
+        if not layer.tags:
+            self.logger.debug("Layer %s has no tags, there's nothing to dissolve.", layer.name)
+            return
+        layer_path = layer.path(self._weights_dir)
+        layer_paths = layer.paths(self._weights_dir)
 
-            self.logger.debug("Dissolving layer from %s to %s.", layer_path, layer_paths)
+        if len(layer_paths) < 2:
+            self.logger.debug("Layer %s has only one texture, skipping.", layer.name)
+            return
 
-            # Check if the image contains any non-zero values, otherwise continue.
-            layer_image = cv2.imread(layer_path, cv2.IMREAD_UNCHANGED)
+        self.logger.debug("Dissolving layer from %s to %s.", layer_path, layer_paths)
+        # Check if the image contains any non-zero values, otherwise continue.
+        layer_image = cv2.imread(layer_path, cv2.IMREAD_UNCHANGED)
+        if layer_image is None:
+            self.logger.debug("Layer %s image not found, skipping.", layer.name)
+            return
 
-            if not np.any(layer_image):  # type: ignore
-                self.logger.debug(
-                    "Layer %s does not contain any non-zero values, skipping.", layer.name
-                )
-                continue
+        # Get mask of non-zero pixels. If there are no non-zero pixels, skip the layer.
+        mask = layer_image > 0
+        if not np.any(mask):
+            self.logger.debug(
+                "Layer %s does not contain any non-zero values, skipping.", layer.name
+            )
+            return
+        # Save the original image to use it for preview later, without combining the sublayers.
+        cv2.imwrite(layer.path_preview(self._weights_dir), layer_image.copy())  # type: ignore
 
-            # Save the original image to use it for preview later, without combining the sublayers.
-            cv2.imwrite(layer.path_preview(self._weights_dir), layer_image.copy())  # type: ignore
+        # Create random assignment array for all pixels
+        random_assignment = np.random.randint(0, layer.count, size=layer_image.shape)
 
-            # Get the coordinates of non-zero values.
-            non_zero_coords = np.column_stack(np.where(layer_image > 0))  # type: ignore
+        # Create sublayers using vectorized operations.
+        sublayers = []
+        for i in range(layer.count):
+            # Create sublayer: 255 where (mask is True AND random_assignment == i)
+            sublayer = np.where((mask) & (random_assignment == i), 255, 0).astype(np.uint8)
+            sublayers.append(sublayer)
 
-            # Prepare sublayers.
-            sublayers = [np.zeros_like(layer_image) for _ in range(layer.count)]
+        # Save sublayers
+        for sublayer, sublayer_path in zip(sublayers, layer_paths):
+            cv2.imwrite(sublayer_path, sublayer)
 
-            # Randomly assign non-zero values to sublayers.
-            for coord in non_zero_coords:
-                sublayers[np.random.randint(0, layer.count)][coord[0], coord[1]] = 255  # type: ignore
-
-            # Save the sublayers.
-            for sublayer, sublayer_path in zip(sublayers, layer_paths):
-                cv2.imwrite(sublayer_path, sublayer)
-                self.logger.debug("Sublayer %s saved.", sublayer_path)
-
-            self.logger.debug("Dissolved layer %s.", layer.name)
+        self.logger.debug("Dissolved layer %s.", layer.name)
 
     def draw_base_layer(self, cumulative_image: np.ndarray) -> None:
         """Draws base layer and saves it into the png file.
@@ -749,6 +805,27 @@ class Texture(ImageComponent):
         ox_settings.use_cache = self.map.texture_settings.use_cache
         ox_settings.requests_timeout = 30
 
+        objects = self.fetch_osm_data(tags)
+        if objects is None or objects.empty:
+            self.logger.debug("No objects found for tags: %s.", tags)
+            return
+
+        self.logger.debug("Fetched %s elements for tags: %s.", len(objects), tags)
+
+        method = self.linestrings_generator if yield_linestrings else self.polygons_generator
+
+        yield from method(objects, width, is_fieds)
+
+    @monitor_performance
+    def fetch_osm_data(self, tags: dict[str, str | list[str] | bool]) -> gpd.GeoDataFrame | None:
+        """Fetches OSM data for given tags.
+
+        Arguments:
+            tags (dict[str, str | list[str] | bool]): Dictionary of tags to search for.
+
+        Returns:
+            gpd.GeoDataFrame | None: GeoDataFrame with OSM objects or None if no objects found.
+        """
         try:
             if self.map.custom_osm is not None:
                 with warnings.catch_warnings():
@@ -758,20 +835,17 @@ class Texture(ImageComponent):
                 objects = ox.features_from_bbox(bbox=self.new_bbox, tags=tags)
         except Exception as e:
             self.logger.debug("Error fetching objects for tags: %s. Error: %s.", tags, e)
-            return
-        self.logger.debug("Fetched %s elements for tags: %s.", len(objects), tags)
+            return None
 
-        method = self.linestrings_generator if yield_linestrings else self.polygons_generator
-
-        yield from method(objects, width, is_fieds)
+        return objects
 
     def linestrings_generator(
-        self, objects: pd.core.frame.DataFrame, *args, **kwargs
+        self, objects: gpd.GeoDataFrame, *args, **kwargs
     ) -> Generator[list[tuple[int, int]], None, None]:
         """Generator which yields lists of point coordinates which represent LineStrings from OSM.
 
         Arguments:
-            objects (pd.core.frame.DataFrame): Dataframe with OSM objects.
+            objects (gpd.GeoDataFrame): GeoDataFrame with OSM objects.
 
         Yields:
             Generator[list[tuple[int, int]], None, None]: List of point coordinates.
@@ -817,6 +891,7 @@ class Texture(ImageComponent):
             polygon_np = self._to_np(polygon)
             yield polygon_np
 
+    @monitor_performance
     def previews(self) -> list[str]:
         """Invokes methods to generate previews. Returns list of paths to previews.
 
