@@ -9,41 +9,40 @@ from osmnx import settings as ox_settings
 
 from maps4fs.logger import Logger
 
+TQDM_DISABLE = os.getenv("TQDM_DISABLE", "0") == "1"
+
 logger = Logger()
 
 MFS_TEMPLATES_DIR = os.path.join(os.getcwd(), "templates")
+
 MFS_DEFAULTS_DIR = os.path.join(os.getcwd(), "defaults")
 MFS_DEM_DEFAULTS_DIR = os.path.join(MFS_DEFAULTS_DIR, "dem")
 MFS_OSM_DEFAULTS_DIR = os.path.join(MFS_DEFAULTS_DIR, "osm")
-os.makedirs(MFS_OSM_DEFAULTS_DIR, exist_ok=True)
-os.makedirs(MFS_DEM_DEFAULTS_DIR, exist_ok=True)
+MFS_MSETTINGS_DEFAULTS_DIR = os.path.join(MFS_DEFAULTS_DIR, "main_settings")
+MFS_GSETTINGS_DEFAULTS_DIR = os.path.join(MFS_DEFAULTS_DIR, "generation_settings")
+default_dirs = [
+    MFS_DEM_DEFAULTS_DIR,
+    MFS_OSM_DEFAULTS_DIR,
+    MFS_MSETTINGS_DEFAULTS_DIR,
+    MFS_GSETTINGS_DEFAULTS_DIR,
+]
+for directory in default_dirs:
+    os.makedirs(directory, exist_ok=True)
+
 logger.info("MFS_TEMPLATES_DIR: %s. MFS_DEFAULTS_DIR: %s.", MFS_TEMPLATES_DIR, MFS_DEFAULTS_DIR)
+logger.info(
+    "MFS_DEM_DEFAULTS_DIR: %s. MFS_OSM_DEFAULTS_DIR: %s. "
+    "MFS_MSETTINGS_DEFAULTS_DIR: %s. MFS_GSETTINGS_DEFAULTS_DIR: %s.",
+    MFS_DEM_DEFAULTS_DIR,
+    MFS_OSM_DEFAULTS_DIR,
+    MFS_MSETTINGS_DEFAULTS_DIR,
+    MFS_GSETTINGS_DEFAULTS_DIR,
+)
 
-DEFAULT_OSM_PATH = os.path.join(MFS_OSM_DEFAULTS_DIR, "custom_osm.osm")
-DEFAULT_DEM_PATH = os.path.join(MFS_DEM_DEFAULTS_DIR, "custom_dem.png")
-logger.info("DEFAULT_OSM_PATH: %s. DEFAULT_DEM_PATH: %s.", DEFAULT_OSM_PATH, DEFAULT_DEM_PATH)
-
-
-def default_osm() -> str | None:
-    """Get the path to the default OSM file if it exists.
-
-    Returns:
-        str | None: The path to the default OSM file, or None if it doesn't exist.
-    """
-    res = DEFAULT_OSM_PATH if os.path.isfile(DEFAULT_OSM_PATH) else None
-    logger.info("Default OSM in %s, result: %s", DEFAULT_OSM_PATH, res)
-    return res
-
-
-def default_dem() -> str | None:
-    """Get the path to the default DEM file if it exists.
-
-    Returns:
-        str | None: The path to the default DEM file, or None if it doesn't exist.
-    """
-    res = DEFAULT_DEM_PATH if os.path.isfile(DEFAULT_DEM_PATH) else None
-    logger.info("Default DEM in %s, result: %s", DEFAULT_DEM_PATH, res)
-    return res
+TEMPLATES_STRUCTURE = {
+    "fs25": ["texture_schemas", "tree_schemas", "map_templates"],
+    "fs22": ["texture_schemas", "map_templates"],
+}
 
 
 def ensure_templates():
@@ -130,7 +129,19 @@ def ensure_templates():
         raise
 
 
+def ensure_template_subdirs() -> None:
+    """Ensure that all expected subdirectories exist in the templates directory."""
+    for game_version, subdirs in TEMPLATES_STRUCTURE.items():
+        for subdir in subdirs:
+            dir_path = os.path.join(MFS_TEMPLATES_DIR, game_version, subdir)
+            if not os.path.exists(dir_path):
+                logger.debug("Expected template subdirectory missing: %s", dir_path)
+                os.makedirs(dir_path, exist_ok=True)
+    logger.info("Templates directory is ready at: %s", MFS_TEMPLATES_DIR)
+
+
 ensure_templates()
+ensure_template_subdirs()
 
 MFS_ROOT_DIR = os.getenv("MFS_ROOT_DIRECTORY", os.path.join(os.getcwd(), "mfsrootdir"))
 MFS_CACHE_DIR = os.path.join(MFS_ROOT_DIR, "cache")
