@@ -6,6 +6,7 @@ from __future__ import annotations
 
 import os
 
+import maps4fs.generator.config as mfscfg
 from maps4fs.generator.component.background import Background
 from maps4fs.generator.component.config import Config
 from maps4fs.generator.component.grle import GRLE
@@ -13,8 +14,6 @@ from maps4fs.generator.component.i3d import I3d
 from maps4fs.generator.component.satellite import Satellite
 from maps4fs.generator.component.texture import Texture
 
-working_directory = os.getcwd()
-
 
 class Game:
     """Class used to define different versions of the game for which the map is generated.
@@ -34,22 +33,44 @@ class Game:
     code: str | None = None
     dem_multipliyer: int = 2
     _additional_dem_name: str | None = None
-    _map_template_path: str | None = None
-    _texture_schema: str | None = None
-    _grle_schema: str | None = None
-    _tree_schema: str | None = None
+    _map_template_file: str | None = None
+    _texture_schema_file: str | None = None
+    _grle_schema_file: str | None = None
+    _tree_schema_file: str | None = None
     _i3d_processing: bool = True
     _plants_processing: bool = True
     _environment_processing: bool = True
     _fog_processing: bool = True
     _dissolve: bool = True
+    _mesh_processing: bool = True
 
     # Order matters! Some components depend on others.
-    components = [Texture, Background, GRLE, I3d, Config, Satellite]
+    components = [Satellite, Texture, Background, GRLE, I3d, Config]
 
     def __init__(self, map_template_path: str | None = None):
         if map_template_path:
             self._map_template_path = map_template_path
+        else:
+            if not self._map_template_file:
+                raise ValueError("Map template file not set.")
+            self._map_template_path = os.path.join(
+                mfscfg.MFS_TEMPLATES_DIR, self._map_template_file  # type: ignore
+            )
+
+        if not self._texture_schema_file:
+            self._texture_schema = None
+        else:
+            self._texture_schema = os.path.join(mfscfg.MFS_TEMPLATES_DIR, self._texture_schema_file)  # type: ignore
+
+        if not self._grle_schema_file:
+            self._grle_schema = None
+        else:
+            self._grle_schema = os.path.join(mfscfg.MFS_TEMPLATES_DIR, self._grle_schema_file)  # type: ignore
+
+        if not self._tree_schema_file:
+            self._tree_schema = None
+        else:
+            self._tree_schema = os.path.join(mfscfg.MFS_TEMPLATES_DIR, self._tree_schema_file)  # type: ignore
 
     def set_components_by_names(self, component_names: list[str]) -> None:
         """Sets the components used for map generation by their names.
@@ -159,6 +180,16 @@ class Game:
             str: The path to the weights directory."""
         raise NotImplementedError
 
+    def license_plates_dir_path(self, map_directory: str) -> str:
+        """Returns the path to the license plates directory.
+
+        Arguments:
+            map_directory (str): The path to the map directory.
+
+        Returns:
+            str: The path to the license plates directory."""
+        raise NotImplementedError
+
     def get_density_map_fruits_path(self, map_directory: str) -> str:
         """Returns the path to the density map fruits file.
 
@@ -233,6 +264,16 @@ class Game:
             str: The path to the i3d file."""
         raise NotImplementedError
 
+    def overview_file_path(self, map_directory: str) -> str:
+        """Returns the path to the overview image file.
+
+        Arguments:
+            map_directory (str): The path to the map directory.
+
+        Returns:
+            str: The path to the overview image file."""
+        raise NotImplementedError
+
     @property
     def i3d_processing(self) -> bool:
         """Returns whether the i3d file should be processed.
@@ -292,18 +333,27 @@ class Game:
             bool: True if the dissolve should be applied, False otherwise."""
         return self._dissolve
 
+    @property
+    def mesh_processing(self) -> bool:
+        """Returns whether the mesh should be processed.
+
+        Returns:
+            bool: True if the mesh should be processed, False otherwise."""
+        return self._mesh_processing
+
 
 class FS22(Game):
     """Class used to define the game version FS22."""
 
     code = "FS22"
-    _map_template_path = os.path.join(working_directory, "data", "fs22-map-template.zip")
-    _texture_schema = os.path.join(working_directory, "data", "fs22-texture-schema.json")
+    _map_template_file = "fs22-map-template.zip"
+    _texture_schema_file = "fs22-texture-schema.json"
     _i3d_processing = False
     _environment_processing = False
     _fog_processing = False
     _plants_processing = False
     _dissolve = False
+    _mesh_processing = False
 
     def dem_file_path(self, map_directory: str) -> str:
         """Returns the path to the DEM file.
@@ -342,10 +392,10 @@ class FS25(Game):
     code = "FS25"
     dem_multipliyer: int = 2
     _additional_dem_name = "unprocessedHeightMap.png"
-    _map_template_path = os.path.join(working_directory, "data", "fs25-map-template.zip")
-    _texture_schema = os.path.join(working_directory, "data", "fs25-texture-schema.json")
-    _grle_schema = os.path.join(working_directory, "data", "fs25-grle-schema.json")
-    _tree_schema = os.path.join(working_directory, "data", "fs25-tree-schema.json")
+    _map_template_file = "fs25-map-template.zip"
+    _texture_schema_file = "fs25-texture-schema.json"
+    _grle_schema_file = "fs25-grle-schema.json"
+    _tree_schema_file = "fs25-tree-schema.json"
 
     def dem_file_path(self, map_directory: str) -> str:
         """Returns the path to the DEM file.
@@ -378,6 +428,16 @@ class FS25(Game):
             str: The path to the weights directory."""
         return os.path.join(map_directory, "map", "data")
 
+    def license_plates_dir_path(self, map_directory: str) -> str:
+        """Returns the path to the license plates directory.
+
+        Arguments:
+            map_directory (str): The path to the map directory.
+
+        Returns:
+            str: The path to the license plates directory."""
+        return os.path.join(map_directory, "map", "licensePlates")
+
     def i3d_file_path(self, map_directory: str) -> str:
         """Returns the path to the i3d file.
 
@@ -407,3 +467,13 @@ class FS25(Game):
         Returns:
             str: The path to the environment xml file."""
         return os.path.join(map_directory, "map", "config", "environment.xml")
+
+    def overview_file_path(self, map_directory: str) -> str:
+        """Returns the path to the overview image file.
+
+        Arguments:
+            map_directory (str): The path to the map directory.
+
+        Returns:
+            str: The path to the overview image file."""
+        return os.path.join(map_directory, "map", "overview.dds")
