@@ -5,6 +5,7 @@ from __future__ import annotations
 import json
 import os
 import shutil
+from time import perf_counter
 from typing import Any, Generator
 
 from pydtmdl import DTMProvider
@@ -12,10 +13,15 @@ from pydtmdl.base.dtm import DTMProviderSettings
 
 import maps4fs.generator.config as mfscfg
 import maps4fs.generator.utils as mfsutils
-from maps4fs.generator.component import Background, Component, Layer, Texture
+from maps4fs.generator.component import Background, Component, Layer, Satellite, Texture
 from maps4fs.generator.game import Game
+from maps4fs.generator.monitor import PerformanceMonitor, performance_session
 from maps4fs.generator.settings import GenerationSettings, MainSettings, SharedSettings
-from maps4fs.generator.statistics import send_advanced_settings, send_main_settings
+from maps4fs.generator.statistics import (
+    send_advanced_settings,
+    send_main_settings,
+    send_performance_report,
+)
 from maps4fs.logger import Logger
 
 
@@ -66,10 +72,23 @@ class Map:
         # endregion
 
         # region custom OSM properties
-        self.custom_osm = custom_osm or mfscfg.default_osm()
         if custom_osm and not os.path.isfile(custom_osm):
             raise FileNotFoundError(f"Custom OSM file {custom_osm} does not exist.")
-        mfsutils.check_and_fix_osm(self.custom_osm, save_directory=self.map_directory)
+        mfsutils.check_and_fix_osm(custom_osm, save_directory=self.map_directory)
+        self.custom_osm = custom_osm
+        # endregion
+
+        # region custom DEM handling
+        self.custom_background_path: str | None = None
+        custom_dem = kwargs.get("custom_background_path", None)
+        if custom_dem and not os.path.isfile(custom_dem):
+            raise FileNotFoundError(f"Custom DEM file {custom_dem} does not exist.")
+
+        # Make a copy of the custom DEM to the map directory.
+        if custom_dem:
+            save_path = os.path.join(self.map_directory, os.path.basename(custom_dem))
+            shutil.copyfile(custom_dem, save_path)
+            self.custom_background_path = save_path
         # endregion
 
         # region main settings
@@ -116,18 +135,32 @@ class Map:
         # endregion
 
         # region prepare map working directory
+        custom_template_path = kwargs.get("custom_template_path", None)
+        if custom_template_path:
+            self.logger.info("Using custom map template: %s", custom_template_path)
+            if not os.path.isfile(custom_template_path):
+                self.logger.error(
+                    "Custom map template file %s does not exist.", custom_template_path
+                )
+                raise FileNotFoundError(
+                    f"Custom map template file {custom_template_path} does not exist."
+                )
+            template_path = custom_template_path
+        else:
+            self.logger.info("Using default map template: %s", game.template_path)
+            template_path = game.template_path
         try:
-            shutil.unpack_archive(game.template_path, self.map_directory)
+            shutil.unpack_archive(template_path, self.map_directory)
             self.logger.debug("Map template unpacked to %s", self.map_directory)
         except Exception as e:
             raise RuntimeError(f"Can not unpack map template due to error: {e}") from e
         # endregion
 
+        self.assets_directory = os.path.join(self.map_directory, "assets")
+        os.makedirs(self.assets_directory, exist_ok=True)
+
         self.shared_settings = SharedSettings()
         self.components: list[Component] = []
-        custom_dem = kwargs.get("custom_background_path", None)
-        self.custom_background_path = custom_dem or mfscfg.default_dem()
-        self.logger.info("Custom DEM path: %s", self.custom_background_path)
 
     def process_settings(self) -> None:
         """Checks the settings by predefined rules and updates them accordingly."""
@@ -176,51 +209,76 @@ class Map:
         Yields:
             Generator[str, None, None]: Component names.
         """
-        self.logger.debug(
-            "Starting map generation. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
-            self.game.code,
-            self.coordinates,
-            self.size,
-            self.rotation,
-        )
-        for game_component in self.game.components:
-            component = game_component(
-                self.game,
-                self,
+        with performance_session() as session_id:
+            self.logger.info(
+                "Starting map generation. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
+                self.game.code,
                 self.coordinates,
                 self.size,
-                self.rotated_size,
                 self.rotation,
-                self.map_directory,
-                self.logger,
-                texture_custom_schema=self.texture_custom_schema,  # type: ignore
-                tree_custom_schema=self.tree_custom_schema,  # type: ignore
             )
-            self.components.append(component)
+            generation_start = perf_counter()
 
-            yield component.__class__.__name__
-
-            try:
-                component.process()
-                component.commit_generation_info()
-            except Exception as e:
-                self.logger.error(
-                    "Error processing or committing generation info for component %s: %s",
-                    component.__class__.__name__,
-                    e,
+            for game_component in self.game.components:
+                component = game_component(
+                    self.game,
+                    self,
+                    self.coordinates,
+                    self.size,
+                    self.rotated_size,
+                    self.rotation,
+                    self.map_directory,
+                    self.logger,
+                    texture_custom_schema=self.texture_custom_schema,  # type: ignore
+                    tree_custom_schema=self.tree_custom_schema,  # type: ignore
                 )
-                self._update_main_settings({"error": str(e)})
-                raise e
+                self.components.append(component)
 
-        self._update_main_settings({"completed": True})
+                yield component.__class__.__name__
 
-        self.logger.debug(
-            "Map generation completed. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
-            self.game.code,
-            self.coordinates,
-            self.size,
-            self.rotation,
-        )
+                try:
+                    component_start = perf_counter()
+                    component.process()
+                    component_finish = perf_counter()
+                    self.logger.info(
+                        "Component %s processed in %.2f seconds.",
+                        component.__class__.__name__,
+                        component_finish - component_start,
+                    )
+                    component.commit_generation_info()
+                except Exception as e:
+                    self.logger.error(
+                        "Error processing or committing generation info for component %s: %s",
+                        component.__class__.__name__,
+                        e,
+                    )
+                    self._update_main_settings({"error": str(e)})
+                    raise e
+
+            generation_finish = perf_counter()
+            self.logger.info(
+                "Map generation completed in %.2f seconds.",
+                generation_finish - generation_start,
+            )
+
+            self._update_main_settings({"completed": True})
+
+            self.logger.info(
+                "Map generation completed. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
+                self.game.code,
+                self.coordinates,
+                self.size,
+                self.rotation,
+            )
+
+        session_json = PerformanceMonitor().get_session_json(session_id)
+        if session_json:
+            report_filename = "performance_report.json"
+            with open(
+                os.path.join(self.map_directory, report_filename), "w", encoding="utf-8"
+            ) as file:
+                json.dump(session_json, file, indent=4)
+            send_performance_report(session_json)
 
     def _update_main_settings(self, data: dict[str, Any]) -> None:
         """Update main settings with provided data.
@@ -277,6 +335,17 @@ class Map:
             return None
         return component
 
+    def get_satellite_component(self) -> Satellite | None:
+        """Get satellite component.
+
+        Returns:
+            Satellite | None: Satellite instance or None if not found.
+        """
+        component = self.get_component("Satellite")
+        if not isinstance(component, Satellite):
+            return None
+        return component
+
     def get_texture_layer(self, by_usage: str | None = None) -> Layer | None:
         """Get texture layer by usage.
 
