@@ -28,12 +28,19 @@ class Parameters:
     WATER = "water"
     FARMYARDS = "farmyards"
 
+    MAXIMUM_BACKGROUND_TEXTURE_SIZE = 4096
+
     PREVIEW_MAXIMUM_SIZE = 2048
 
     BACKGROUND_DISTANCE = 2048
     FULL = "FULL"
     PREVIEW = "PREVIEW"
 
+    DECIMATED_BACKGROUND = "decimated_background"
+    BACKGROUND_TERRAIN = "background_terrain"
+
+    WATER_RESOURCES = "water_resources"
+
     RESIZE_FACTOR = 8
 
     FARMLAND_ID_LIMIT = 254
@@ -48,6 +55,9 @@ class Parameters:
 
     HEIGHT_SCALE = "heightScale"
 
+    OVERVIEW_IMAGE_SIZE = 4096
+    OVERVIEW_IMAGE_FILENAME = "overview"
+
 
 class SharedSettings(BaseModel):
     """Represents the shared settings for all components."""
@@ -226,6 +236,7 @@ class I3DSettings(SettingsModel):
             existing points.
         add_reversed_splines (bool): if True, reversed splines will be added to the map.
         field_splines (bool): if True, splines will be added to the fields.
+        license_plate_prefix (str): prefix for the license plates.
     """
 
     add_trees: bool = True
@@ -237,6 +248,8 @@ class I3DSettings(SettingsModel):
     add_reversed_splines: bool = False
     field_splines: bool = False
 
+    license_plate_prefix: str = "M4S"
+
 
 class TextureSettings(SettingsModel):
     """Represents the advanced settings for texture component.
@@ -323,6 +336,7 @@ class MainSettings(NamedTuple):
     rotation: int
     dtm_provider: str
     custom_osm: bool
+    custom_dem: bool
     is_public: bool
     date: str
     time: str
@@ -358,6 +372,7 @@ class MainSettings(NamedTuple):
             "rotation": self.rotation,
             "dtm_provider": self.dtm_provider,
             "custom_osm": self.custom_osm,
+            "custom_dem": self.custom_dem,
             "is_public": self.is_public,
             "date": self.date,
             "time": self.time,
@@ -388,6 +403,7 @@ class MainSettings(NamedTuple):
             rotation=map.rotation,
             dtm_provider=map.dtm_provider.name(),
             custom_osm=bool(map.custom_osm),
+            custom_dem=bool(map.custom_background_path),
             is_public=map.kwargs.get("is_public", False),
             date=datetime.now().strftime("%Y-%m-%d"),
             time=datetime.now().strftime("%H:%M:%S"),
